@@ -6,54 +6,38 @@ use Digest::MD5;
 use HTTP::Response;
 use base 'LWP::UserAgent';
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 sub new {
 	my ($class, %opts) = @_;
 	
-	my $cache_dir = delete $opts{cache_dir};
-	my $nocache   = delete $opts{nocache};
-	my $recache   = delete $opts{recache};
+	my $cache_dir      = delete $opts{cache_dir};
+	my $nocache_if     = delete $opts{nocache_if};
+	my $recache_if     = delete $opts{recache_if};
+	my $cachename_spec = delete $opts{cachename_spec};
 	my $self = $class->SUPER::new(%opts);
 	
-	$self->{cache_dir} = $cache_dir;
-	$self->{nocache}   = $nocache;
-	$self->{recache}   = $recache;
+	$self->{cache_dir}      = $cache_dir;
+	$self->{nocache_if}     = $nocache_if;
+	$self->{recache_if}     = $recache_if;
+	$self->{cachename_spec} = $cachename_spec;
 	
 	return $self;
 }
 
-sub cache_dir {
-	my $self = shift;
-	if (@_) {
-		my $cache_dir = $self->{cache_dir};
-		$self->{cache_dir} = shift;
-		return $cache_dir;
+# generate getters and setters
+foreach my $opt_name (qw(cache_dir nocache_if recache_if cachename_spec)) {
+	no strict 'refs';
+	*$opt_name = sub {
+		my $self = shift;
+		if (@_) {
+			my $opt_val = $self->{$opt_name};
+			$self->{$opt_name} = shift;
+			return $opt_val;
+		}
+		
+		return $self->{$opt_name};
 	}
-	
-	return $self->{cache_dir};
-}
-
-sub nocache {
-	my $self = shift;
-	if (@_) {
-		my $nocache = $self->{nocache};
-		$self->{nocache} = shift;
-		return $nocache;
-	}
-	
-	return $self->{nocache};
-}
-
-sub recache {
-	my $self = shift;
-	if (@_) {
-		my $recache = $self->{recache};
-		$self->{recache} = shift;
-		return $recache;
-	}
-	
-	return $self->{recache};
 }
 
 sub simple_request {
@@ -64,7 +48,7 @@ sub simple_request {
 	
 	my $request = $_[0];
 	eval{ $self->prepare_request($request) };
-	my $fpath = $self->{cache_dir} . '/' . Digest::MD5::md5_hex($request->as_string);
+	my $fpath = $self->_get_cache_name($request);
 	my $response;
 	my $no_collision_suffix;
 	
@@ -88,8 +72,8 @@ sub simple_request {
 			}
 		}
 		
-		if ($response && defined($self->{recache}) && ref($self->{recache}) eq 'CODE' &&
-		    $self->{recache}->($response, $fpath))
+		if ($response && defined($self->{recache_if}) && ref($self->{recache_if}) eq 'CODE' &&
+		    $self->{recache_if}->($response, $fpath))
 		{
 			$response = undef;
 		}
@@ -98,7 +82,7 @@ sub simple_request {
 	unless ($response) {
 		$response = $self->SUPER::simple_request(@_);
 		
-		if (!defined($self->{nocache}) || ref($self->{nocache}) ne 'CODE' || !$self->{nocache}->($response)) {
+		if (!defined($self->{nocache_if}) || ref($self->{nocache_if}) ne 'CODE' || !$self->{nocache_if}->($response)) {
 			if (defined $no_collision_suffix) {
 				$fpath .= $no_collision_suffix;
 			}
@@ -123,11 +107,51 @@ sub simple_request {
 	return $response;
 }
 
+sub last_cached {
+	my $self = shift;
+	return exists $self->{last_cached} ?
+		@{$self->{last_cached}} : ();
+}
+
 sub uncache {
 	my $self = shift;
-	if (exists $self->{last_cached}) {
-		unlink $_ for @{$self->{last_cached}};
+	unlink $_ for $self->last_cached;
+}
+
+sub _get_cache_name {
+	my ($self, $request) = @_;
+	
+	if (defined($self->{cachename_spec}) && %{$self->{cachename_spec}}) {
+		my $tmp_request = $request->clone();
+		my $leave_only_specified;
+		if (exists $self->{cachename_spec}{_headers}) {
+			ref $self->{cachename_spec}{_headers} eq 'ARRAY'
+				or croak 'cachename_spec->{_headers} should be array ref';
+			$leave_only_specified = 1;
+		}
+		
+		foreach my $hname ($tmp_request->headers->header_field_names) {
+			if (exists $self->{cachename_spec}{$hname}) {
+				if (defined $self->{cachename_spec}{$hname}) {
+					$tmp_request->headers->header($hname, $self->{cachename_spec}{$hname});
+				}
+				else {
+					$tmp_request->headers->remove_header($hname);
+				}
+			}
+			elsif ($leave_only_specified && !_in($hname, $self->{cachename_spec}{_headers})) {
+				$tmp_request->headers->remove_header($hname);
+			}
+		}
+		
+		if (exists $self->{cachename_spec}{_body}) {
+			$tmp_request->content($self->{cachename_spec}{_body});
+		}
+		
+		return $self->{cache_dir} . '/' . Digest::MD5::md5_hex($tmp_request->as_string);
 	}
+	
+	return $self->{cache_dir} . '/' . Digest::MD5::md5_hex($request->as_string);
 }
 
 sub _parse_cached_response {
@@ -226,7 +250,7 @@ seems it may cache responses and get responses from the cache, but has too much
 
 All LWP::UserAgent methods and few new.
 
-=head2 new(cache_dir => , nocache => , recache => , ...)
+=head2 new(...)
 
 Creates new LWP::UserAgent::Cached object. Since LWP::UserAgent::Cached is LWP::UserAgent subclass it has all same
 parameters, but in additional it has some new optional pararmeters:
@@ -234,37 +258,96 @@ parameters, but in additional it has some new optional pararmeters:
 cache_dir - Path to the directory where cache will be stored. If not set useragent will behaves as LWP::UserAgent without
 cache support.
 
-nocache - Reference to subroutine. First parameter of this subroutine will be HTTP::Response object. This subroutine
+nocache_if - Reference to subroutine. First parameter of this subroutine will be HTTP::Response object. This subroutine
 should return true if this response should not be cached and false otherwise. If not set all responses will be cached.
 
-recache - Reference to subroutine. First parameter of this subroutine will be HTTP::Response object, second - path to
-file with cache. This subroutine should return true if response needs to be recached (new HTTP request will be made)
+recache_if - Reference to subroutine. First parameter of this subroutine will be HTTP::Response object, second - path to
+file with cache. This subroutine should return true if response needs to be recache_ifd (new HTTP request will be made)
 and false otherwise. This subroutine will be called only if response already available in the cache.
 
-Example:
+cachename_spec - Hash reference to cache naming specification. In fact cache naming for each request based on request content.
+Internally it is md5_hex($request->as_string). But what if some of request headers in your program changed dinamically, e.g.
+User-Agent or Cookie? In such case caching will not work properly for you. We need some way to omit this headers when calculating
+cache name. This option is what you need. Specification hash should contain header name and header value which will be used 
+(instead of values in request) while calculating cache name.
+
+For example we already have cache where 'User-Agent' value in the headers was 'Mozilla/5.0', but in the current version of the program 
+it will be changed for each request. So we force specified that for cache name calculation 'User-Agent' should be 'Mozilla/5.0'. Cached
+request had not 'Accept' header, but in the current version it has. So we force specified do not include this header for cache name
+calculation.
+
+    cachename_spec => {
+        'User-Agent' => 'Mozilla/5.0',
+        'Accept' => undef
+    }
+
+Specification hash may contain two special keys: '_body' and '_headers'. With '_body' key you can specify body content in the request
+for cache name calculation. For example to not include body content in cache name calculation set '_body' to undef or empty string.
+With '_headers' key you can specify which headers should be included in $request for cache name calculation. For example you can say to
+include only 'Host' and 'Referer'. '_headers' value should be array reference:
+
+    cachename_spec => {
+        _body => undef, # omit body
+        _headers => ['Host'], # include only host with value from request
+        # It will be smth like:
+        # md5_hex("METHOD url\r\nHost: host\r\n\r\n")
+        # method and url will be included in any case
+    }
+
+Another example. Omit body, include only 'Host' and 'User-Agent' headers, use 'Host' value from request and specified 'User-Agent' value,
+in addition include referrer with specified value ('Referer' not specified in '_headers', but values from main specification hash has
+higher priority):
+
+    cachename_spec => {
+        _body => '',
+        _headers => ['Host', 'User-Agent'],
+        'User-Agent' => 'Mozilla/5.0',
+        'Referer' => 'http://www.com'
+    }
+
+One more example. Calculate cache name based only on method and url:
+
+    cachename_spec => {
+        _body =>'',
+        _headers => []
+    }
+
+LWP::UserAgent::Cached creation example:
 
     use LWP::UserAgent::Cached;
     
-    my $ua = LWP::UserAgent::Cached->new(cache_dir => 'cache/lwp', nocache => sub {
+    my $ua = LWP::UserAgent::Cached->new(cache_dir => 'cache/lwp', nocache_if => sub {
         my $response = shift;
         return $response->code >= 500; # do not cache any bad response
-    }, recache => sub {
+    }, recache_if => sub {
         my ($response, $path) = @_;
-        return $response->code == 404 && -M $path > 1 # recache any 404 response older than 1 day
+        return $response->code == 404 && -M $path > 1 # recache_if any 404 response older than 1 day
+    },
+    cachename_spec => {
+        'User-Agent' => undef, # omit agent while calculating cache name
     });
 
 =head2 cache_dir() or cache_dir($dir)
 
 Gets or sets corresponding option from the constructor.
 
-=head2 nocache() or nocache($sub)
+=head2 nocache_if() or nocache_if($sub)
 
 Gets or sets corresponding option from the constructor.
 
-=head2 recache() or recache($sub)
+=head2 recache_if() or recache_if($sub)
 
 Gets or sets corresponding option from the constructor.
 
+=head2 cachename_spec() or cachename_spec($spec)
+
+Gets or sets corresponding option from the constructor.
+
+=head2 last_cached()
+
+Returns list with pathes to files with cache stored by last noncached response. List may contain more than one
+element if there was redirect.
+
 =head2 uncache()
 
 Removes last response from the cache. Use case example:
